#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
"""Unittest for testing the update_version file"""

import logging
import unittest
from pathlib import Path
from sys import stdout


class TestUpdateVersion(unittest.TestCase):

    def setUp(self) -> None:
        """Run before every test method"""
        # define a format
        custom_format = '[%(asctime)s] [%(levelname)-8s] [%(filename)-15s @'\
                        ' %(funcName)-15s:%(lineno)4s] %(message)s'

        # set basic config and level for all loggers
        logging.basicConfig(level=logging.INFO,
                            format=custom_format,
                            stream=stdout)

        # create a logger for this TestSuite
        self.test_logger = logging.getLogger(__name__)
        self.package_logger = logging.getLogger('UpdateVersion')

        # set the test logger level
        self.test_logger.setLevel(logging.DEBUG)
        self.package_logger.setLevel(logging.DEBUG)

        self._here = Path(__file__).parent

    def tearDown(self) -> None:
        """Run after every test method"""
        pass

    @unittest.skip("Not yet implemented")
    def test_parser_valid_file(self) -> None:
        pass

    @unittest.skip("Not yet implemented")
    def test_validate_regex(self) -> None:
        pass

    @unittest.skip("Not yet implemented")
    def test_parse_arguments(self) -> None:
        pass

    @unittest.skip("Not yet implemented")
    def test_create_version_info_line(self) -> None:
        pass

    @unittest.skip("Not yet implemented")
    def test_update_version_file(self) -> None:
        pass


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-

"""
Extract a single version line from a changelog and extract the semver content
from this line
"""

import json
import logging
import re
from pathlib import Path
from sys import stdout
from typing import Dict, List, Optional

from semver import VersionInfo


class ExtractVersionError(Exception):
    """Base class for exceptions in this module."""
    pass


class ExtractVersion(object):
    """Extract the version line and SemVer part from a changelog file"""
    def __init__(self, logger: Optional[logging.Logger] = None):
        """
        Init ExtractVersion class

        :param      logger:             Logger object
        :type       logger:             Optional[logging.Logger]
        """
        if logger is None:
            logger = self._create_logger()
        self._logger = logger
        self._semver_data = VersionInfo(*(0, 0, 0))
        self._latest_description_lines = []
        self._meta_data = {}

        self._semver_line_regex = (
            r"^(?P<major>0|[1-9]\d*)\."     # major version part
            r"(?P<minor>0|[1-9]\d*)\."      # minor version part
            r"(?P<patch>0|[1-9]\d*)"        # bugfix/patch version part
            # optional prerelease version part, starting with a "-"
            r"(?:-(?P<prerelease>(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?"    # noqa
            # optional build metadata version part, starting with a "+"
            r"(?:\+(?P<buildmetadata>[0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$"
        )

        self._version_line_regex = (
            # begin of line with two "#" followed by a single space
            r"(?P<title_begin>\#\#)[ ]{1}"
            # anything after a "["
            r"\[(?P<potential_semver>("
            # three numbers with one or more digits, seperated by dot
            r"(\d{1,}\.\d{1,}\.\d{1,})"
            r"([-+]?)"  # zero or one of either a "-" or "+" character
            # r"(.*)"  # any character zero or more times
            r"([a-zA-Z.+-d]*)"  # any character (a-Z), dot, "-", "+" or number
                                # for zero or more times
            r")(?=\]))\]"    # positive lookahead for the "]" and the "]"
            r"[ ]{1}\-[ ]{1}"     # exactly one space, "-", exactly one space
            r"(?P<datetime>\d{4}\-\d{2}-\d{2})"     # datetime as YYYY-MM-DD
            r"(([T ]{1})"   # seperation between date and time by "T" or space
            r"(?P<timestamp>\d{2,}:\d{2,}:\d{2,}?))?"   # time as HH:MM:SS
        )

        self._date_line_regex = (
            r".*"    # anything
            r"(?P<datetime>\d{4}\-\d{2}-\d{2})"     # datetime as YYYY-MM-DD
            r"(([T ]{1})"   # seperation between date and time by "T" or space
            r"(?P<timestamp>\d{2,}:\d{2,}:\d{2,}?))?"   # time as HH:MM:SS
        )

    @property
    def version_line_regex(self) -> str:
        """
        Get regex to extract complete version line from changelog

        :returns:   Regex of the complete version line
        :rtype:     str
        """
        return self._version_line_regex

    @version_line_regex.setter
    def version_line_regex(self, value: str) -> None:
        """
        Set regex to extract complete version line from changelog

        :param      value:  Regex to get the complete version line
        :type       value:  str
        """
        try:
            re.compile(value)
            self._version_line_regex = value
        except re.error:
            raise ExtractVersionError("Invalid regex pattern")

    @property
    def semver_line_regex(self) -> str:
        """
        Get regex to extract the semver part from the complete version line

        :returns:   Regex of the semver part
        :rtype:     str
        """
        return self._semver_line_regex

    @semver_line_regex.setter
    def semver_line_regex(self, value: str) -> None:
        """
        Set regex to extract the semver part from the complete version line

        :param      value:  Regex to get the semver part
        :type       value:  str
        """
        try:
            re.compile(value)
            self._semver_line_regex = value
        except re.error:
            raise ExtractVersionError("Invalid regex pattern")

    @property
    def date_line_regex(self) -> str:
        """
        Get regex to extract the date part from the complete version line

        :returns:   Regex to get the date part
        :rtype:     str
        """
        return self._date_line_regex

    @date_line_regex.setter
    def date_line_regex(self, value: str) -> None:
        """
        Set regex to extract the date part from the complete version line

        :param      value:  Regex to get the date part
        :type       value:  str
        """
        try:
            re.compile(value)
            self._date_line_regex = value
        except re.error:
            raise ExtractVersionError("Invalid regex pattern")

    @property
    def semver_data(self) -> VersionInfo:
        """
        Get VersionInfo object with latest SemVer data

        :returns:   The version information
        :rtype:     VersionInfo
        """
        return self._semver_data

    @semver_data.setter
    def semver_data(self, value: VersionInfo) -> None:
        """
        Set SemVer data

        :param      value:  The value
        :type       value:  VersionInfo
        """
        if isinstance(value, VersionInfo):
            self._semver_data = value
        else:
            raise ExtractVersionError("Value is not of type VersionInfo")

    @property
    def latest_description_lines(self) -> List[str]:
        """
        Get latest description lines of the parsed changelog

        :returns:   Content of latest release
        :rtype:     List[str]
        """
        return self._latest_description_lines

    @property
    def latest_description(self) -> str:
        """
        Get latest description of the parsed changelog

        :returns:   Latest release description
        :rtype:     str
        """
        return '\n'.join(self.latest_description_lines)

    @property
    def meta_data(self) -> Dict[str, str]:
        """
        Get latest meta data of the parsed changelog

        :returns:   Latest meta data
        :rtype:     str
        """
        return self._meta_data

    @staticmethod
    def _create_logger(logger_name: str = None) -> logging.Logger:
        """
        Create a logger

        :param      logger_name:  The logger name
        :type       logger_name:  str, optional

        :returns:   Configured logger
        :rtype:     logging.Logger
        """
        custom_format = '[%(asctime)s] [%(levelname)-8s] [%(filename)-15s @'\
                        ' %(funcName)-15s:%(lineno)4s] %(message)s'

        # configure logging
        logging.basicConfig(level=logging.INFO,
                            format=custom_format,
                            stream=stdout)

        if logger_name and (isinstance(logger_name, str)):
            logger = logging.getLogger(logger_name)
        else:
            logger = logging.getLogger(__name__)

        # set the logger level to DEBUG if specified differently
        logger.setLevel(logging.DEBUG)

        return logger

    def parse_changelog(self, changelog_file: Path) -> str:
        """
        Parse the changelog for the first matching version line

        :param      changelog_file:  The path to the changelog file
        :type       changelog_file:  Path

        :returns:   Extracted semantic version string
        :rtype:     str
        """
        release_version_line = ""

        release_version_lines = self.parse_changelog_completely(
            changelog_file=changelog_file,
            first_line_only=True)

        if len(release_version_lines) >= 1:
            release_version_line = release_version_lines[0]

        return release_version_line

    def parse_changelog_completely(self,
                                   changelog_file: Path,
                                   first_line_only: bool = False) -> List[str]:
        """
        Parse the changelog for all matching version lines

        :param      changelog_file:  The path to the changelog file
        :type       changelog_file:  Path
        :param      first_line_only: Flag to break after first match found
        :type       first_line_only: bool

        :returns:   List of extracted semantic version strings
        :rtype:     List[str]
        """
        release_version_lines = []
        matches_found = 0
        latest_description_lines = []

        with open(changelog_file, "r") as f:
            for line in f:
                match = re.search(self.version_line_regex, line)
                if match:
                    release_version_lines.append(match.group())
                    matches_found += 1

                    if first_line_only:
                        break

                # collect the lines until the next (second) match is found
                if matches_found == 1:
                    latest_description_lines.append(line.strip())

        self._logger.debug("Matching release version lines: '{}'".
                           format(release_version_lines))
        self._logger.debug("Latest description lines: '{}'".
                           format(latest_description_lines[1:]))

        # the version line itself is also included, ignore it
        self._latest_description_lines = latest_description_lines[1:]

        self.parse_meta_comment()

        return release_version_lines

    def parse_semver_line_date(self, release_version_line: str) -> str:
        """
        Parse a version line for a valid ISO8601 datetime

        Examples of a valid ISO8601 datetime lines:
        - "## [0.2.0] - 2022-05-19"
        - "## [107.3.18] - 1900-01-01 12:34:56"
        - "## [1.0.0-alpha-a.b-c-somethinglong+build.1-aef.1-its-okay] - 2012-01-02"    # noqa

        :param      release_version_line:  The release version line
        :type       release_version_line:  str

        :returns:   ISO8601 datetime string, e.g. "1970-01-01"
        :rtype:     str
        """
        date_string = "1970-01-01"

        match = re.search(self.date_line_regex, release_version_line)
        if match:
            if len(match.groups()) >= 4 and match.group(2):
                date_string = match.group(1) + match.group(2)
            else:
                date_string = match.group(1)

        return date_string

    def parse_semver_line(self, release_version_line: str) -> str:
        """
        Parse a version line for a semantic version

        Examples of a valid SemVer line:
        - "## [0.2.0] - 2022-05-19"
        - "## [107.3.18] - 1900-01-01 12:34:56"
        - "## [1.0.0-alpha-a.b-c-somethinglong+build.1-aef.1-its-okay] - 2012-01-02"    # noqa

        :param      release_version_line:  The release version line
        :type       release_version_line:  str

        :returns:   Semantic version string, e.g. "0.2.0"
        :rtype:     str
        """
        semver_string = "0.0.0"

        # try to extract any content between square brackets
        match = re.search(r"\[(.*?)\]", release_version_line)
        if not match:
            return semver_string

        # the potential semver content is the first group of the complete line
        potential_semver = match.group(1)

        # try to extract semver from release version line
        match = re.search(self.semver_line_regex, potential_semver)

        if match:
            semver_string = match.group()
            if not VersionInfo.isvalid(semver_string):
                self._logger.error("Parsed SemVer string is invalid, check "
                                   "the changelog format")
                raise ValueError("Invalid SemVer string")
            self._logger.debug("Extracted SemVer string: '{}'".
                               format(semver_string))
            self.semver_data = VersionInfo.parse(semver_string)
        else:
            self._logger.warning("No SemVer string found in given release "
                                 "version line: '{}'".
                                 format(release_version_line))

        return semver_string

    def parse_meta_comment(self) -> None:
        """Find and parse meta comment line of all parsed description lines"""
        for line in self.latest_description_lines:
            # try to extract any comment with "meta ="
            match = re.search(r"(<!--\smeta\s=\s)(.*?)(\s-->)", line)

            if match and len(match.groups()) == 3:
                self._meta_data = json.loads(match.groups()[1].replace("'", "\""))  # noqa: E501
                self._logger.debug("Meta Data: '{}'".format(self._meta_data))
                break



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-

"""Render version file based on template"""

import logging
from pathlib import Path
from typing import Optional, Union

from jinja2 import Environment, FileSystemLoader

from .extract_version import ExtractVersion


class RenderVersionFileError(Exception):
    """Base class for exceptions in this module."""
    pass


class RenderVersionFile(object):
    """docstring for RenderVersionFile"""
    def __init__(self,
                 template_path: Optional[Path] = None,
                 logger: Optional[logging.Logger] = None):
        """
        Init RenderVersionFile class

        :param      template_path: Path to templates
        :type       template_path: Path
        :param      logger:        Logger object
        :type       logger:        Optional[logging.Logger]
        """
        if logger is None:
            logger = ExtractVersion._create_logger()
        self._logger = logger

        self._env = None
        self._default_template_path = Path(__file__).parent / "templates"
        self._content = ""

    @property
    def default_template_path(self) -> Path:
        """
        Get path to default template folder

        :returns:   Path to template folder
        :rtype:     Path
        """
        return self._default_template_path

    @default_template_path.setter
    def default_template_path(self, template_path: Union[Path, str]) -> None:
        """
        Set path to default template folder

        :param      template_path:  The path to the template folder
        :type       template_path:  Union[Path, str]
        """
        template_path = Path(template_path)
        if template_path.exists():
            if template_path.is_dir():
                self._default_template_path = template_path
            else:
                raise RenderVersionFileError(
                    "Template path can only be a directory")
        else:
            raise RenderVersionFileError(
                "Specified directory '{}' doesn't exist".format(template_path))

    @property
    def content(self) -> str:
        """
        Get rendered version file content

        :returns:   Rendered version file content
        :rtype:     str
        """
        return self._content

    def _find_file(self, template: Union[Path, str]) -> Path:
        """
        Find template file on disk or in package templates directory

        :param      template:  The path to the template file
        :type       template:  Union[Path, str]

        :returns:   Resolved file location
        :rtype:     Path
        """
        template = Path(template)
        template_path = ""

        if template.exists():
            self._logger.debug("Template '{}' found".format(template))
            # check if file exists as the user specified it
            if template.is_file():
                template_path = template.parent
            elif template.is_dir():
                raise RenderVersionFileError(
                    "Can not render a directory, please specify a single "
                    "template file")
        elif (self.default_template_path / template).exists():
            # check if file might exist in the package templates directory
            self._logger.debug("Template '{}' found in package templates '{}'".
                               format(template, self.default_template_path))
            template = self.default_template_path / template
            if template.is_file():
                template_path = template.parent
            elif template.is_dir():
                raise RenderVersionFileError(
                    "Can not render a directory, please specify a single "
                    "template file")
        else:
            self._logger.error(
                "Template '{}' neither found in package templates directory "
                "'{}' nor at the specified path".
                format(template, self.default_template_path))
            raise RenderVersionFileError(
                "Template path/file '{}' does not exist".format(template))

        self._logger.debug("Using template path: {}".format(template_path))

        self._env = Environment(loader=FileSystemLoader(template_path),
                                keep_trailing_newline=True)

        return template.resolve()

    def render_file(self,
                    file_path: Path,
                    content: dict,
                    template: Union[Path, str],
                    save_file: bool = True) -> None:
        """
        Render a template file with given content

        :param      file_path   The path to the file
        :type       file_path:  Path
        :param      content:    The content
        :type       content:    dict
        :param      template:   The path to the template file
        :type       template:   Union[Path, str]
        :param      save_file:  Save rendered content to file
        :type       save_file:  bool
        """
        template_file = self._find_file(template=template)

        content["file_name"] = file_path.name
        content["file_name_without_suffix"] = file_path.stem
        content["template_name"] = template_file.name
        content["template_name_without_suffix"] = template_file.stem

        file_template = self._env.get_template(template_file.name)
        rendered_content = file_template.render(content)
        self._content = rendered_content

        if not save_file:
            return

        Path(file_path.parent).mkdir(parents=True, exist_ok=True)

        if file_path.exists():
            self._logger.info("Overwriting file '{}'".format(file_path))

        with open(file_path, "w") as file:
            file.write(rendered_content)



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-

"""
Update version info in template file with latest changelog version

The changelog and package version shall always be aligned. This script can be
used to render a python version file from the latest changelog entry.
This changelog entry shall follow the semantic version pattern, see
https://semver.org/ and shall match the following pattern:

## [x.y.z] - yyyy-mm-dd

The line shall start with two hashtags followed by a single space. The semver
with x, y and z as non-negative integers, seperated by a dot and surrounded by
square brackets. Followed by a space, a dash, another space and the ISO8601
formatted date. Additional timestamps after the data, seperated from the date
by a single space or a capital "T", are optional.
The semantic version tag inside the square brackets supports the full scope.
"""

import argparse
import fileinput
import json
import logging
import re
from hashlib import sha1
from pathlib import Path
from sys import stdout

import semver

from .extract_version import ExtractVersion
from .render_version_file import RenderVersionFile
from .version import __version__


def parser_valid_file(parser: argparse.ArgumentParser, arg: str) -> Path:
    """
    Determine whether file exists.
    :param      parser:                 The parser
    :type       parser:                 parser object
    :param      arg:                    The file to check
    :type       arg:                    str
    :raise      argparse.ArgumentError: Argument is not a file
    :returns:   Input file path, parser error is thrown otherwise.
    :rtype:     Path
    """
    if not Path(arg).is_file():
        parser.error("The file {} does not exist!".format(arg))
    else:
        return Path(arg).resolve()


def validate_regex(parser: argparse.ArgumentParser, arg: str) -> str:
    """
    Validate given regex pattern
    :param      parser:                 The parser
    :type       parser:                 parser object
    :param      arg:                    The regex pattern to check
    :type       arg:                    str
    :raise      argparse.ArgumentError: Argument is not a file
    :returns:   Regex pattern, parser error is thrown otherwise.
    :rtype:     str
    """
    try:
        re.compile(arg)
    except re.error:
        parser.error("The regex pattern '{}' is invalid".format(arg))
    return arg


def parse_arguments() -> argparse.Namespace:
    """
    Parse CLI arguments.
    :raise      argparse.ArgumentError  Argparse error
    :return:    argparse object
    """
    parser = argparse.ArgumentParser(description="""
    Update version info file based on changelog entry
    """, formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    # default arguments
    parser.add_argument('-d', '--debug',
                        action='store_true',
                        help='Output logger messages to stderr')
    parser.add_argument('-v',
                        default=0,
                        action='count',
                        dest='verbosity',
                        help='Set level of verbosity, default is CRITICAL')
    parser.add_argument('--version',
                        action='version',
                        version='%(prog)s {version}'.
                                format(version=__version__),
                        help="Print version of package and exit")

    # specific arguments
    parser.add_argument('--changelog_file',
                        dest='changelog_file',
                        required=True,
                        type=lambda x: parser_valid_file(parser, x),
                        help='Path to changelog file')

    parser.add_argument('--version_file',
                        dest='version_file',
                        required=False,
                        help='Path to rendered file')

    parser.add_argument('--version_file_type',
                        dest='version_file_type',
                        required=False,
                        choices=['py', 'c'],
                        default='py',
                        type=lambda x: x.lower(),
                        help='Type of version file to generate')

    parser.add_argument('--template_file',
                        dest='template_file',
                        required=False,
                        type=lambda x: parser_valid_file(parser, x),
                        help='Path to template version file')

    parser.add_argument('--additional_template_data',
                        dest='additional_template_data',
                        required=False,
                        type=json.loads,
                        help='Additional data as JSON to render the template')

    parser.add_argument('--additional_version_info',
                        dest='additional_version_info',
                        required=False,
                        type=str,
                        help='Additional version informations like "-rc1234"')

    parser.add_argument('--version_line_regex',
                        dest='version_line_regex',
                        required=False,
                        type=lambda x: validate_regex(parser, x),
                        help='Regex to extract complete version line from a '
                             'changelog')

    parser.add_argument('--semver_line_regex',
                        dest='semver_line_regex',
                        required=False,
                        type=lambda x: validate_regex(parser, x),
                        help='Regex to extract semver part of from a version '
                             'line')

    parser.add_argument('--output',
                        dest='dump_to_file',
                        required=False,
                        help='Dump parsed changelog as JSON file to file')

    parser.add_argument('--validate',
                        dest='do_validate',
                        required=False,
                        action='store_true',
                        help='Validate existing version file with changelog based version')  # noqa: E501

    parser.add_argument('--print',
                        dest='print_result',
                        required=False,
                        action='store_true',
                        help='Print parsed changelog as JSON to stdout')

    parser.add_argument('--pretty',
                        dest='pretty_output',
                        action='store_true',
                        help='Print JSON data at stdout in readable format')

    parsed_args = parser.parse_args()

    return parsed_args


def main():
    # parse CLI arguments
    args = parse_arguments()

    log_levels = {
        0: logging.CRITICAL,
        1: logging.ERROR,
        2: logging.WARNING,
        3: logging.INFO,
        4: logging.DEBUG,
    }
    custom_format = '[%(asctime)s] [%(levelname)-8s] [%(filename)-15s @'\
                    ' %(funcName)-15s:%(lineno)4s] %(message)s'
    logging.basicConfig(level=logging.INFO,
                        format=custom_format,
                        stream=stdout)
    logger = logging.getLogger(__name__)
    logger.setLevel(level=log_levels[min(args.verbosity,
                                     max(log_levels.keys()))])
    logger.disabled = not args.debug

    # changelog_file = Path(args.changelog_file).resolve()
    changelog_file = args.changelog_file
    version_file = None
    template_file = args.template_file
    version_file_type = args.version_file_type
    additional_template_data = args.additional_template_data
    additional_version_info = args.additional_version_info
    version_line_regex = args.version_line_regex
    semver_line_regex = args.semver_line_regex
    dump_to_file = args.dump_to_file
    do_validate = args.do_validate
    print_result = args.print_result
    pretty_output = args.pretty_output

    if args.version_file:
        version_file = Path(args.version_file).resolve()
        logger.debug("Using changelog file '{}' to update version file '{}'".
                     format(changelog_file, version_file))

    version_extractor = ExtractVersion(logger=logger)

    if semver_line_regex:
        logger.debug("Use this regex to get the semver part from the "
                     "version line: {}".format(semver_line_regex))
        version_extractor.semver_line_regex = semver_line_regex

    if version_line_regex:
        logger.debug("Use this regex to get the version line from the "
                     "changelog file: {}".format(version_line_regex))
        version_extractor.version_line_regex = version_line_regex

    version_line = version_extractor.parse_changelog(
        changelog_file=changelog_file)
    version_lines = version_extractor.parse_changelog_completely(
        changelog_file=changelog_file)

    release_infos = {}
    for line in version_lines:
        this_semver_string = version_extractor.parse_semver_line(
            release_version_line=line)
        this_date_string = version_extractor.parse_semver_line_date(
            release_version_line=line)
        release_infos[this_semver_string] = [{"upload_time": this_date_string}]

    semver_string = version_extractor.parse_semver_line(
        release_version_line=version_line)

    file_renderer = RenderVersionFile()
    semver_data = version_extractor.semver_data
    additional_data = ""
    if additional_version_info:
        additional_data = " + '{}'".format(additional_version_info)
    version_file_content = {
        "major_version": semver_data.major,
        "minor_version": semver_data.minor,
        "patch_version": semver_data.patch,
        "prerelease_data": semver_data.prerelease,
        "build_data": semver_data.build,
        "additional_data": additional_data,
    }
    if additional_template_data:
        version_file_content.update(additional_template_data)

    if not template_file:
        # no template file specified, use package template file
        template_file_map = {
            "py": "version.py.template",
            "c": "version.h.template",
        }

        if version_file_type in template_file_map:
            template_file = template_file_map[version_file_type]
            logger.debug("Selected '{}' based on version_file_type: '{}'".
                         format(template_file, version_file_type))
        else:
            raise KeyError("Either specify a custom template file or choose"
                           "a template from this list: {}".
                           format(template_file_map.keys()))

    if do_validate:
        file_renderer.render_file(
            template=template_file,
            file_path=version_file,
            content=version_file_content,
            save_file=False
        )

        file_sha1 = sha1(version_file.read_bytes()).hexdigest()
        rendered_sha1 = sha1(file_renderer.content.encode()).hexdigest()

        if file_sha1 != rendered_sha1:
            raise SystemExit(
                'Mismatch between version file and latest changelog version'
            )

    if version_file and not do_validate:
        file_renderer.render_file(
            template=template_file,
            file_path=version_file,
            content=version_file_content
        )

    changelog_data = {
        'info': {
            'version': semver_string,
            'description': version_extractor.latest_description,
            'meta': version_extractor.meta_data,
        },
        'releases': release_infos
    }

    if print_result:
        if pretty_output:
            stdout.write(json.dumps(changelog_data, indent=4))
        else:
            stdout.write(json.dumps(changelog_data))

    if dump_to_file:
        with open(dump_to_file, 'w') as file:
            if pretty_output:
                file.write(json.dumps(changelog_data, indent=4))
            else:
                file.write(json.dumps(changelog_data))


if __name__ == '__main__':
    main()
